#!/bin/sh
#
# rc.S:  System initialization script that runs at the very beginning of booting.
# This is the only script that run when the system boot as single user mode
# boot: linux single  or linux 1
# Think this as a failsave booting. Thus it only setup:
# swap, local file system, minimum hardware, and minimum local setting
# 
# Originally from Slackware 9.0
# Mostly written by:  Patrick J. Volkerding, <volkerdi@ftp.cdrom.com>
#
# Adapted for Vector Linux by Eko M. Budi
# Initializes:
#  - kernel automatic file system (devfs, dev, proc, etc)
#  - swap  
#  - root file system
#  - local file system
#  - clean file system
#  - set basic hardware, just enough for single user
#  - set basic settings

. /etc/rc.d/functions-display

PATH=/sbin:/usr/sbin:/bin:/usr/bin
dmesg -n 1

#Set YES if you want a "rainbow-technicolor" initialization
export  RCINITCOLORS="YES" # {YES|NO}

echoc "==> rc.S  Setting up the base system configuration....." yellow
#sleep 2

echoc "Initialising kernel file systems" cyan

# Mount /proc right away:
echo "Mounting /proc file system" 
/sbin/mount -v proc /proc -n -t proc

# If 'nohotplug' was given at boot, or rc.hotplug has been turned off
# (is not executable), then shut off hotplugging in the kernel now.
if [ -w /proc/sys/kernel/hotplug ]; then
  if grep nohotplug /proc/cmdline 1> /dev/null 2> /dev/null ; then
    echo "Turning hotplug OFF" 
    echo "/dev/null" > /proc/sys/kernel/hotplug
  elif [ ! -x /etc/rc.d/rc.hotplug ]; then
    echo "Turning hotplug OFF"
    echo "/dev/null" > /proc/sys/kernel/hotplug
  fi
fi

## Start devfsd if necessary.
## On kernels 2.6 or newer, udev should be used instead.
if [ ! -x /etc/rc.d/rc.udev ]; then
  if [ -r /dev/.devfsd ] && [ -x /sbin/devfsd ]; then
      echo "Starting devfs daemon:  /sbin/devfsd /dev" 
      /sbin/devfsd /dev
  fi
fi

# Mount sysfs next, if the kernel supports it:
if [ -d /sys ]; then
  if cat /proc/filesystems | grep -w sysfs 1> /dev/null 2> /dev/null ; then
    if ! cat /proc/mounts | grep -w sysfs 1> /dev/null 2> /dev/null ; then
      echo "Starting sysfs"
      /sbin/mount -v sysfs /sys -n -t sysfs
    fi
  fi
fi

# Initialize udev to manage /dev entries for 2.6.x kernels:
if [ -x /etc/rc.d/rc.udev ]; then
  if ! grep -w nohotplug /proc/cmdline 1> /dev/null 2> /dev/null ; then
     echo "Starting udev"
    /etc/rc.d/rc.udev  
  fi
fi

##-------------------------------------------------------
## SWAP and LOCAL FILE SYSTEM

## Start swap and update
echoc "Activating swap file" cyan
swapon -a 
/sbin/update &
swapon -s

##--------------------------------------------------------
# Start file system
echoc "Mounting local file systems" cyan

# Test to see if the root partition is read-only, like it ought to be.
READWRITE=no
if echo -n >> "Testing filesystem status"; then
 rm -f "Testing filesystem status"
 READWRITE=yes
fi

# See if a forced filesystem check was requested at shutdown:
if [ -r /etc/forcefsck ]; then
  FORCEFSCK="-f"
fi

# Check the root filesystem:
if [ ! $READWRITE = yes ]; then
  if [ ! -r /etc/fastboot ]; then
    echo "Checking root filesystem:"
    /sbin/fsck $FORCEFSCK -C -a /
    RETVAL=$?
  fi
  # An error code of 2 or higher will require a reboot.
  if [ $RETVAL -ge 2 ]; then
    # An error code equal to or greater than 4 means that some errors
    # could not be corrected.  This requires manual attention, so we
    # offer a chance to try to fix the problem in single-user mode:
    if [ $RETVAL -ge 4 ]; then
      echo
      echo
      echo "***********************************************************"
      echo "*** An error occurred during the root filesystem check. ***"
      echo "*** You will now be given a chance to log into the      ***"
      echo "*** system in single-user mode to fix the problem.      ***"
      echo "***                                                     ***"
      echo "*** If you are using the ext2 filesystem, running       ***"
      echo "*** 'e2fsck -v -y <partition>' might help.              ***"
      echo "***********************************************************"
      echo
      echo "Once you exit the single-user shell, the system will reboot."
      echo
      PS1="(Repair filesystem) \#"; export PS1
      sulogin
    else # With an error code of 2 or 3, reboot the machine automatically:
      echo
      echo
      echo "***********************************"
      echo "*** The filesystem was changed. ***"
      echo "*** The system will now reboot. ***"
      echo "***********************************"
      echo
    fi
    echo "Unmounting file systems."
    umount -a -r
    mount -n -o remount,ro /
    echo "Rebooting system."
    sleep 2
    reboot -f
  fi
  # Remount the root filesystem in read-write mode
  echo "Remounting root device with read-write enabled."
  /sbin/mount -w -v -n -o remount /
  if [ $? -gt 0 ] ; then
    echo
    echo "Attempt to remount root device as read-write failed!  This is going to"
    echo "cause serious problems."
    echo 
    echo "If you're using the UMSDOS filesystem, you **MUST** mount the root partition"
    echo "read-write!  You can make sure the root filesystem is getting mounted "
    echo "read-write with the 'rw' flag to Loadlin:"
    echo
    echo "loadlin vmlinuz root=/dev/hda1 rw   (replace /dev/hda1 with your root device)"
    echo
    echo "Normal bootdisks can be made to mount a system read-write with the rdev command:"
    echo
    echo "rdev -R /dev/fd0 0"
    echo
    echo "You can also get into your system by using a boot disk with a command like this"
    echo "on the LILO prompt line:  (change the root partition name as needed)"
    echo 
    echo "LILO: mount root=/dev/hda1 rw"
    echo
    echo "Please press ENTER to continue, then reboot and use one of the above methods to"
    echo -n "get into your machine and start looking for the problem. " 
    read junk; 
  fi
else
  echo "Testing filesystem status: read-write filesystem"
  if cat /etc/fstab | grep ' / ' | grep umsdos 1> /dev/null 2> /dev/null ; then
    ROOTTYPE="umsdos"
  fi
  if [ ! "$ROOTTYPE" = "umsdos" ]; then # no warn for UMSDOS
    echo
    echo "*** ERROR: Root partition has already been mounted read-write. Cannot check!"
    echo
    echo "For filesystem checking to work properly, your system must initially mount"
    echo "the root partition as read only. Please modify your kernel with 'rdev' so that"
    echo "it does this. If you're booting with LILO, add a line:"
    echo
    echo "   read-only"
    echo
    echo "to the Linux section in your /etc/lilo.conf and type 'lilo' to reinstall it."
    echo
    echo "If you boot from a kernel on a floppy disk, put it in the drive and type:"
    echo "   rdev -R /dev/fd0 1"
    echo
    echo "If you boot from a bootdisk, or with Loadlin, you can add the 'ro' flag."
    echo
    echo "This will fix the problem *AND* eliminate this annoying message. :^)"
    echo
    echo -n "Press ENTER to continue. "
    read junk;
  fi
fi # Done checking root filesystem

# Any /etc/mtab that exists here is old, so we delete it to start over:
/bin/rm -f /etc/mtab*
# Remounting the / partition will initialize the new /etc/mtab:
/sbin/mount -w -o remount /

#--------------------------------------------------------------------------------
# Additional file system now

# Initialize the Logical Volume Manager.
# This won't start unless we find /etc/lvmtab (LVM1) or 
# /etc/lvm/backup/ (LVM2).  This is created by /sbin/vgscan, so to
# use LVM you must run /sbin/vgscan yourself the first time (and
# create some VGs and LVs).
if [ -r /etc/lvmtab -o -d /etc/lvm/backup ]; then
  echo "Initializing LVM (Logical Volume Manager):"
  # Check for device-mapper support.
  if ! cat /proc/devices | grep -w device-mapper 1> /dev/null 2> /dev/null ; then
    # If device-mapper exists as a module, try to load it.
    if [ -r /lib/modules/$(cat /proc/sys/kernel/osrelease)/kernel/drivers/md/dm-mod.ko ]; then
      insmod /lib/modules/$(cat /proc/sys/kernel/osrelease)/kernel/drivers/md/dm-mod.ko
    fi
  fi
  # Scan for new volume groups:
  /sbin/vgscan 2> /dev/null
  if [ $? = 0 ]; then
    # This needs a moment to register.
    sleep 10
    # Make volume groups available to the kernel.
    # This should also make logical volumes available.
    /sbin/vgchange -ay
    # Enable swapping again in case any LVs are used for swap.  Ignore previous error.  :-)
    /sbin/swapon -a
  fi
fi

# Check all the non-root filesystems:
if [ ! -r /etc/fastboot ]; then
  echo "Checking non-root filesystems:"
  /sbin/fsck $FORCEFSCK -C -R -A -a 
fi

## mount file systems in fstab (and create an entry for /)
## but not NFS,SAMBA because TCP/IP is not yet configured
echo "Mounting local file systems"
/sbin/mount -a -v -t nonfs,nosmbfs,noproc 

## Clearing local file system.
## Note that if you mount /tmp or /var from NFS/SAMBA later,
## do these again. OK ?
echoc "Resetting the system so the game can start " cyan

# Remove undesired things
echo "Removing stale lock, temporary and junk files"

# Clean up some temporary files:
( cd /var/log/setup/tmp && rm -rf * )
/bin/rm -f /var/run/utmp /var/run/*pid /etc/nologin /var/run/lpd* \
  /var/run/ppp* /etc/dhcpc/dhcpcd-eth0.pid /etc/forcefsck /etc/fastboot

# remove /etc/mtab* so that mount will create it with a root entry
/bin/rm -f  /etc/nologin /etc/shutdownpid /etc/dhcpc/*.pid 

# Clean up temporary files on the /var volume:
/bin/rm -f /var/run/utmp /var/run/*.pid /var/log/setup/tmp/*

# Remove stale locks and junk files 
/bin/rm -f /var/lock/* /var/spool/uucp/LCK..* /tmp/.X*lock /tmp/core /core 1> /dev/null 2> /dev/null

# Remove stale hunt sockets.
if [ -r /tmp/hunt -o -r /tmp/hunt.stats ]; then
  /bin/rm -f /tmp/hunt*
fi

# Remove vasm files
/bin/rm -f /tmp/menu*
/bin/rm -f /tmp/reply*

## Ensure basic filesystem permissions sanity.
echo "Ensuring basic filesystem permissions sanity" 
chmod 755 /
chmod 1777 /tmp /var/tmp

## Looks like we have to create this.
mkdir -p /var/log/lastlog

## Create a fresh utmp file:
touch /var/run/utmp
chown root.utmp /var/run/utmp
chmod 664 /var/run/utmp

## Syncronizing if we run from MSDOS
if [ "$ROOTTYPE" = "umsdos" ]; then # we need to update any files added in DOS:
  echo "Synchronizing UMSDOS directory structure:"
  echo "  /sbin/umssync -r99 -v- /"
  /sbin/umssync -r99 -v- /
fi

## Filesystem is done

## Reset boot log
echo "Initialising boot.log"
echo "Vector Linux Boot Process" > $INITLOG
echo "Boot : " `date` >> $INITLOG
dmesg >> $INITLOG

#----------------------------------------------------------------------------
# BASIC HARDWARE

## Very basic hardware, just enough to setup the local filesystem
##
echocl "Initializing basic hardware ..." cyan

# Configure ISA Plug-and-Play devices:
if [ -x /etc/rc.d/rc.pnp ]; then
    echol "Configuring ISA PNP"
    /etc/rc.d/rc.pnp  >> $INITLOG  2>&1
fi

# Set the system time from the hardware clock using hwclock --hctosys.
# Detect SGI Visual Workstation, since hwclock will make those freeze up:
if fgrep -l Cobalt-APIC /proc/interrupts 1> /dev/null 2> /dev/null ; then
  echol "SGI Visual Workstation detected.  Not running hwclock."
elif [ -x /sbin/hwclock ]; then
  if grep "^UTC" /etc/hardwareclock 1> /dev/null 2> /dev/null ; then
    echol "Setting system time from the hardware clock (UTC)."
    /sbin/hwclock --utc --hctosys   >> $INITLOG 2>&1
  else
    echo "Setting system time from the hardware clock (localtime)."
    /sbin/hwclock --localtime --hctosys  >> $INITLOG 2>&1
  fi
fi

# Carry an entropy pool between reboots to improve randomness.
# Load and then save 512 bytes, which is the size of the entropy pool.
if [ -f /etc/random-seed ]; then
  echol "Using /etc/random-seed to initialize /dev/urandom."
  cat /etc/random-seed >/dev/urandom
fi
dd if=/dev/urandom of=/etc/random-seed count=1 bs=512 2> /dev/null

## Load a custom keymap and console font.
if [ -x /etc/rc.d/rc.keymap ] ; then
  echol "Setting up keymap"
  . /etc/rc.d/rc.keymap  >> $INITLOG 2>&1
fi

## Hardware is done, for now
## Will do advanced harware initialisation on rc.M

##-------------------------------------------------

echocl "Initializing basic system settings ..." cyan

# Configure runtime kernel parameters:
if [ -x /sbin/sysctl -a -r /etc/sysctl.conf ]; then
  echol "Setting up kernel run time patameters"
  /sbin/sysctl -e -p /etc/sysctl.conf   >> $INITLOG 2>&1
fi

## Update all the shared library links automatically
echol "Updating shared libraries"
ldconfig   >> $INITLOG 2>&1

## Screen blanks after 15 minutes idle time.
/bin/setterm -blank 0   >> $INITLOG 2>&1

# If there's no /etc/HOSTNAME, fall back on this default:
if [ ! -r /etc/HOSTNAME ]; then
  echo "vector.mydomain.mynet" > /etc/HOSTNAME
fi

# Set the hostname.
HOSTNAME=`cat /etc/HOSTNAME`
echol "Setting hostname: $HOSTNAME"
hostname `echo $HOSTNAME | cut -f1 -d .`   >> $INITLOG 2>&1

# Setup the /etc/issue and /etc/motd to reflect the current kernel level:
# THESE WIPE ANY CHANGES YOU MAKE TO /ETC/ISSUE AND /ETC/MOTD WITH EACH
# BOOT. COMMENT THEM OUT IF YOU WANT TO MAKE CUSTOM VERSIONS.
linux_logo -c -t "Brought to you by Vector-Linux" > /etc/issue
echo "Welcome to $HOSTNAME. Unauthorized access is prohibited !" > /etc/issue.net
echo "`/bin/uname -a | /bin/cut -d\  -f1,3`" > /etc/motd

#echoc "Startup initialisation is done" green
# All done.

